
//c++ -o omr optical_mark_recognition.cpp -std=c++11 `pkg-config --cflags --libs opencv`
#include<iostream>
#include<vector>
#include <algorithm>
#include<opencv2/opencv.hpp>


bool looksLikeBubble(const int & w,
                     const int & h){
    double ratio=static_cast<double>(w)/static_cast<double>(h);
    return (w>=20)&&(h >= 20)&&(ratio>=0.7)&&(w<=100)&&(ratio<=1.4);
}

bool comparePositionOfTwoBoxes(const std::vector<cv::Point> & box1,
                               const std::vector<cv::Point> & box2){
    return box1[0].x<box2[0].x;
}
std::string getInput(){
    std::cout<<"1. test01.jpg "<<std::endl;
    std::cout<<"2. test02.jpg "<<std::endl;
    std::cout<<"3. test03.jpg "<<std::endl;
    std::cout<<"What is your choice? ";
    int uI;
    std::cin>>uI;
    if(uI==2){
        return "test02.jpg";
    }
    if(uI==3){
        return "test03.jpg";
    }
    return "test01.jpg";
}

int main(){
    std::string fileName=getInput();
    cv::Mat imgMatrix;
    
    imgMatrix=cv::imread(fileName,CV_LOAD_IMAGE_COLOR);
    if(!imgMatrix.data){
        std::cout<<"Could not open "<<fileName<<"."<<std::endl;
        return -1;
    }
    std::cout<<"The image has "<<imgMatrix.rows<<" rows and ";
    std::cout<<imgMatrix.cols<<" columns.";
    std::cout<<std::endl;
    std::cout<<std::endl;
    std::cout<<"STEP 1. REMOVE COLOR"<<std::endl;
    std::cout<<std::endl;
    std::cout<<"The result is grayscaleMatrix.";
    std::cout<<std::endl;
    cv::Mat grayscaleMatrix;

    cv::cvtColor(imgMatrix, grayscaleMatrix, cv::COLOR_BGR2GRAY);
    std::cout<<std::endl;
    std::cout<<"Do you wish to put grayscaleMatrix in gray.png? [1=Yes, 0=No] ";
    int answer;
    std::cin>>answer;
    if(answer==1){
        cv::imwrite("gray.png",grayscaleMatrix);
        std::cout<<"Feel free to open gray.png.";
        std::cout<<std::endl;
    }
    std::cout<<std::endl;
    std::cout<<"STEP 2. MAKE BLACK-WHITE NEGATIVE"<<std::endl;
    std::cout<<std::endl;
    std::cout<<"Every color is turned to either black or white."<<std::endl;
    std::cout<<std::endl;
    int cutoff=50;
    std::cout<<"Every pixel darker than "<<cutoff;
    std::cout<<" becomes white. Other pixels become black. "<<std::endl;
    cv::Mat bwMatrix;
    cv::threshold(grayscaleMatrix,bwMatrix,cutoff,255,1);
    std::cout<<std::endl;
    std::cout<<"Do you wish to save this black/white image in bw.png? ";
    std::cout<<"[1=Yes, 0=No] ";
    std::cin>>answer;
    if(answer==1){
        cv::imwrite("bw.png",bwMatrix);
        std::cout<<"Feel free to open bw.png.";
        std::cout<<std::endl;
    }
    std::cout<<std::endl<<std::endl;
    std::cout<<"STEP 3. IDENTIFYING CONTOURS ";
    std::cout<<std::endl<<std::endl;
    
    std::vector<std::vector<cv::Point> > contours;
    
    cv::findContours(bwMatrix,contours,
                     CV_RETR_EXTERNAL,CV_CHAIN_APPROX_SIMPLE);
    std::cout<<"There are "<<contours.size()<<" contours."<<std::endl;
    std::cout<<"Do you wish to see all contours? [1=Yes, 0=No] ";
    std::cin>>answer;
    if(answer==1){
        cv::Mat imgCnt=cv::Mat::zeros(imgMatrix.rows,
                                      imgMatrix.cols, CV_8UC1);;
        for(int i=0;i<contours.size();++i){
            drawContours(imgCnt,contours,i,255,1);
        }
        cv::imwrite("contours.png",imgCnt);
        std::cout<<"Feel free to open contours.png";
        std::cout<<std::endl;
    }
    
    std::cout<<std::endl;
    std::cout<<"STEP 4. IDENTIFY BUBBLES"<<std::endl;
    std::cout<<std::endl;
    std::cout<<"Each of the contours is first put in a rectangular box. ";
    std::cout<<"Those boxes that are or correct size and ";
    std::cout<<"not too high in the paper ";
    std::cout<<"are our bubbles. "<<std::endl;
    cv::Rect boxAroundContour;
    std::vector<std::vector<cv::Point> > bubbleCnts;
    int bWidth, bHeight;
    for(int i=0;i< contours.size();++i){
        boxAroundContour = boundingRect( cv::Mat(contours[i]) );
        bWidth=(boxAroundContour.br()).x-(boxAroundContour.tl()).x;
        bHeight=(boxAroundContour.br()).y-(boxAroundContour.tl()).y;
        
        if(looksLikeBubble(bWidth,bHeight)){
            if((boxAroundContour.br()).y>200){
                bubbleCnts.push_back(contours[i]);
            }
            
        }
    }
    std::cout<<"Identified "<<bubbleCnts.size()<<" bubbles.\n\n";
    std::cout<<"Do you wish to see them? [1=Yes, 0=No] ";
    std::cin>>answer;
    if(answer==1){
        cv::Mat imgQCnt=cv::Mat::zeros(imgMatrix.rows,
                                       imgMatrix.cols, CV_8UC1);;
        for(int i=0;i<bubbleCnts.size();++i){
            drawContours(imgQCnt,bubbleCnts,i,255,1);
        }
        cv::imwrite("bubbles.png",imgQCnt);
        std::cout<<"Saved. Feel free to open and inspect bubbles.png";
        std::cout<<std::endl;
    }

    std::cout<<std::endl;
    std::cout<<"STEP 5. IDENTIFY FILLED BUBBLES\n\n";
    
    std::cout<<"Step 5.1. Draw a box around each bubble.\n\n";
    std::vector<std::vector<cv::Point> > boxesCnts;
    boxesCnts.resize(bubbleCnts.size());
    for(int i=0;i<bubbleCnts.size();++i){
        cv::Rect smallRectangle=boundingRect( cv::Mat(bubbleCnts[i]) );
        boxesCnts[i].resize(4);
        boxesCnts[i][0]=smallRectangle.tl();
        boxesCnts[i][1]=cv::Point(smallRectangle.tl().x,smallRectangle.br().y);
        boxesCnts[i][2]=smallRectangle.br();
        boxesCnts[i][3]=cv::Point(smallRectangle.br().x,smallRectangle.tl().y);
    }
    std::cout<<"Do you wish to the boxes? [1=Yes, 0=No] ";
    std::cin>>answer;
    if(answer==1){
        cv::Mat imgBCnt=cv::Mat::zeros(imgMatrix.rows,
                                       imgMatrix.cols, CV_8UC1);
        for(int i=0;i<bubbleCnts.size();++i){
            drawContours(imgBCnt,boxesCnts,i,255,1);
        }
        cv::imwrite("boxCnts.png",imgBCnt);
        std::cout<<"Saved. Feel free to open and inspect boxCnts.png";
        std::cout<<std::endl;
    }
    std::cout<<std::endl;
    std::cout<<"Step 5.2. Sort the boxes from left to right.\n\n";
    
    
    std::sort(boxesCnts.begin(),boxesCnts.end(),comparePositionOfTwoBoxes);
    
    
    std::cout<<"Step 5.3. Calculate numbers of white points in boxes.\n\n"<<std::endl;
    std::vector<int> whitePointCounts;
    whitePointCounts.resize(boxesCnts.size());
    for(int i=0;i<boxesCnts.size();++i){
        cv::Mat matrixWithBoxFull=cv::Mat::zeros(imgMatrix.rows,
                                                 imgMatrix.cols, CV_8UC1);
        drawContours(matrixWithBoxFull,boxesCnts,i,255,-1);
        cv::Mat answerToASingleQuestionOnly;
        
        cv::bitwise_and(bwMatrix,matrixWithBoxFull,answerToASingleQuestionOnly);
        whitePointCounts[i]=cv::countNonZero(answerToASingleQuestionOnly);
    }
    
    for(int i=0;i<boxesCnts.size();++i){
        std::cout<<"The number of white dots in box "<<i;
        std::cout<<" is "<<whitePointCounts[i]<<"."<<std::endl;
    }
    std::cout<<std::endl<<"Step 5.4. Decide which bubble is filled\n\n";
    std::cout<<"The filled bubble corresponds to the answer: ";
    if(whitePointCounts[0]>whitePointCounts[1]){
        std::cout<<"T ";
    }
    else{
        std::cout<<"F ";
    }
    std::cout<<std::endl<<std::endl;
    
    return 0;
    
}
